<%-- $Id: exception.jsp 4476 2008-09-05 15:14:42Z jari $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
	------------------------------------------------------------------

	All exceptions are directed to this page, by the server configuration
	See <error-page> tag in the /WEB-INF/web.xml file.

	Some exceptions are fowarded to other pages to be able to
	display a more appropriate error message.

	Only severe, unexpected exceptions, are handled by this page,
	which will only display a stack trace!

	@author Nicklas
	@version 2.0
--%>
<%@ page session="false"
	import="net.sf.basedb.core.Application"
	import="net.sf.basedb.core.Config"
	import="net.sf.basedb.core.NotLoggedInException"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.InvalidDataException"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.clients.web.DuplicateWizardException"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="javax.servlet.ServletException"
	import="java.sql.Driver"
	import="java.util.Properties"
	isErrorPage="true"
%>
<%@
	taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%
	final SessionControl sc = Base.getSessionControl(pageContext, false);
	final String reportbuglink = sc == null ? "" : Values.getStringOrNull(sc.getClientDefaultSetting("server.links.reportbug"));
	Throwable ex = exception;
	
	if (ex instanceof org.apache.jasper.JasperException)
	{
		org.apache.jasper.JasperException jex = (org.apache.jasper.JasperException)ex;
		ex = jex.getRootCause();
	}
	
	if (ex instanceof ServletException)
	{
		ServletException sex = (ServletException)ex;
		if (sex.getRootCause() != null) ex = sex.getRootCause();
	}
	
	if (ex instanceof WebException)
	{
		pageContext.forward("/exception/web.jsp");
		return;
	}
	else if (ex instanceof DuplicateWizardException)
	{
		pageContext.forward("/exception/duplicate_wizard.jsp");
		return;
	}
	else if (ex instanceof NotLoggedInException)
	{
		pageContext.forward("/exception/not_logged_in.jsp");
		return;
	}
	else if (ex instanceof PermissionDeniedException)
	{
		pageContext.forward("/exception/permission_denied.jsp");
		return;
	}
	else if (ex instanceof InvalidDataException)
	{
		pageContext.forward("/exception/invalid_data.jsp");
		return;
	}

	final String jdbcDriver = Config.getString("db.driver");
	String driverVersion = "";
	try
	{
		final Driver driver = (Driver)Class.forName(jdbcDriver).newInstance();
		driverVersion = "(version " + driver.getMajorVersion() + "." + driver.getMinorVersion() + ")";
	}
	catch (Throwable t)
	{}	
	final Properties properties = System.getProperties();
	final Runtime runtime = Runtime.getRuntime();
	String exceptionClassName = ex.getClass().getName().replaceAll("net.sf.basedb.core.", "");
%>
<base:page type="default" menu="exception" title="Unexpected error">
<base:head>
	<script language="JavaScript">
	// Hide and display some elements that should not be displayed at all times
	function fixWindow()
	{
		if (history.length <= 1) // >
		{
			Main.hide('goback');
		}
		if (window.top != null && window.top.opener != null)
		{
			// popup window --> Hide menubar
			Main.hide('menubar');
			var width = Main.getWindowWidth();
			var height = Main.getWindowHeight();
			if (height < 600) height = 600;
			if (width < 800) width = 800; // >
			window.resizeTo(width, height);
		}
		else
		{
			// Hide close button
			Main.hide('close');
			// If inside a second frameset, hide menubar
			if (window.parent != window.top)
			{
				Main.hide('menubar');
			}
		}
	}
	</script>
</base:head>
<base:body onload="fixWindow()">
	<base:note type="error" title="<%=exceptionClassName + " on page "+request.getAttribute("javax.servlet.error.request_uri")%>" 
		style="width:90%;" messagestyle="height: 450px;">
		<table border="0" cellpadding="0" cellspacing="0" class="form">
		<tr>
			<td class="prompt">Version</td>
			<td class="info"><%=Application.getVersionString()%></td>
		</tr>
		<tr>
			<td class="prompt">Web server</td>
			<td class="info"><%=application.getServerInfo()%></td>
		</tr>
		<tr>
			<td class="prompt">Database Server</td>
			<td class="info"><%=Application.getDatabaseVersionString()%></td>
		</tr>
		<tr>
			<td class="prompt">Database Dialect</td>
			<td class="info"><%=Config.getString("db.dialect")%></td>
		</tr>
		<tr>
			<td class="prompt">JDBC Driver</td>
			<td class="info"><%=jdbcDriver%> <%=driverVersion%></td>
		</tr>
		<tr>
			<td class="prompt">Java runtime</td>
			<td class="info"><%=properties.getProperty("java.runtime.name")%> 
				(<%=properties.getProperty("java.runtime.version")%>), 
				<a href="<%=properties.getProperty("java.vendor.url")%>" target="_new"><%=properties.getProperty("java.vendor")%></a></td>
		</tr>
		<tr>
			<td class="prompt">Operating system</td>
			<td class="info"><%=properties.getProperty("os.name")%> 
				<%=properties.getProperty("os.arch")%>
				<%=properties.getProperty("os.version")%>
				</td>
		</tr>
		<tr>
			<td class="prompt">Memory</td>
			<td class="info">Total: <%=Values.formatBytes(runtime.totalMemory())%><br>
				Free: <%=Values.formatBytes(runtime.freeMemory()) %><br>
				Max: <%=Values.formatBytes(runtime.maxMemory()) %></td>
		</tr>
		<tr>
			<td class="prompt">Browser</td>
			<td class="info"><%=HTML.encodeTags(request.getHeader("User-Agent"))%></td>
		</tr>
		<tr>
			<td class="prompt">Error message</td>
			<td class="info"><%=HTML.formatLineBreaks(HTML.encodeTags(ex.getMessage(), ""))%></td>
		</tr>
		<tr>
			<td class="prompt">Stack trace</td>
			<td class="info"></td>
		</tr>
		</table>
		
		<%
		StackTraceElement[] st = ex.getStackTrace();
		out.println("<pre>");
		out.println(exceptionClassName);
		int i = 0;
		for (i=0; i < st.length && i < 8; i++)
		{
			out.print("...at ");
			out.println(HTML.encodeTags(st[i].toString().replaceAll("net.sf.basedb.core.", "")));
		}
		out.println("</pre>");
		if ((i < st.length) || (ex.getCause() != null))
		{
			out.println("<pre id=\"error_detail\" style=\"display:none;\">");
			while (ex != null)
			{
				for (int j=i; j < st.length; j++)
				{
					out.print("...at ");
					out.println(HTML.encodeTags(st[j].toString().replaceAll("net.sf.basedb.core.", "")));
				}
				ex = ex.getCause();
				if (ex != null)
				{
					st = ex.getStackTrace();
					i = 0;
					out.print("\nCaused by: "+ex.getClass().getName()+": " + ex.getMessage() + "\n");
				}
			}
			out.println("</pre>");
		}
		%>
		&nbsp;<span id="more_detail">
			<base:icon image="bullet.gif" clazz="" />
			<a href="javascript:Main.show('error_detail');Main.hide('more_detail');"><%=st.length-i%> more...</a>
		</span>
	</base:note>
	<base:buttongroup align="center">
		<base:button id="goback" onclick="<%="history.go(-1)"%>" title="Back"/>
		<base:button id="close" onclick="<%="window.close()"%>" title="Close"/>
		<base:button id="reportbug" onclick="<%="window.open('"+reportbuglink+"','Reportbug')"%>" 
			title="Report bug&hellip;" image="bug.gif"
		visible="<%=HTML.isValidUrl(reportbuglink)%>"/>
	</base:buttongroup>

</base:body>
</base:page>

