<%-- $Id: upload.jsp 4476 2008-09-05 15:14:42Z jari $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
	------------------------------------------------------------------

	This page loads in the hidden frame of the file upload dialog and
	is responsible for the actual file upload. Before starting the upload
	it creates a FileUploadProgress object and stores it in a session
	setting. This object is used to communicate with the visible progress
	page.
	
	@author Nicklas
	@version 2.0
--%>
<%@ page session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Directory"
	import="net.sf.basedb.core.File"
	import="net.sf.basedb.core.FileType"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.Path"
	import="net.sf.basedb.core.SimpleAbsoluteProgressReporter"
	import="net.sf.basedb.util.zip.FileUnpacker"
	import="net.sf.basedb.util.FileUtil"
	import="net.sf.basedb.util.InputStreamSplitter"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.clients.web.fileupload.FileUpload"
	import="net.sf.basedb.clients.web.fileupload.UploadedFile"
	import="net.sf.basedb.clients.web.fileupload.FileUploadProgress"
	import="net.sf.basedb.clients.web.fileupload.UploadAbortedException"
	import="java.io.OutputStream"
	import="java.io.InputStream"
	import="java.util.zip.ZipInputStream"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%
final Item itemType = Item.FILE;
final SessionControl sc = Base.getExistingSessionControl(pageContext, Permission.CREATE, itemType);
final String ID = sc.getId();
final String root = request.getContextPath()+"/";
final int maxTransferRate = Values.getInt(sc.getClientDefaultSetting("server.transferRate"), 0);
final int directoryId = Values.getInt(request.getParameter("directory_id"), SystemItems.getId(Directory.ROOT));
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
String message = null;
String onload = "";
DbControl dc = null;
FileUpload upload = null;
FileUnpacker unpacker = null;
try
{
	upload = new FileUpload(request, maxTransferRate, FileUpload.DEFAULT_TIMEOUT);
	final FileUploadProgress progress = upload.getProgress();
	sc.setSessionSetting("FileUploadProgress", progress);
	final int maxRecent = Base.getMaxRecent(sc);

	dc = sc.newDbControl();
	final Directory directory = Directory.getById(dc, directoryId);
	directory.checkPermission(Permission.WRITE);

	// Parse request to the file upload section
	UploadedFile uploadedFile = upload.next();
	
	// Get all parameters
	boolean checkMd5 = Values.getBoolean(upload.getParameter("check_md5"), false);
	boolean writeProtected = Values.getBoolean(upload.getParameter("write_protected"), false);
	Boolean compress = null; // null == auto by MIME type
	if (Values.getStringOrNull(upload.getParameter("compressed")) != null) 
	{
		compress = Values.getBoolean(upload.getParameter("compressed"));
	}
	int fileTypeId = Values.getInt(upload.getParameter("filetype_id"));
	String description = upload.getParameter("description");
	int zipFormat = Values.getInt(upload.getParameter("zip_format"), 0);
	boolean isZipFile = zipFormat > 0;
	boolean unpackZip = isZipFile && Values.getBoolean(upload.getParameter("zip_unpack"), false);
	boolean overwriteZip = unpackZip && Values.getBoolean(upload.getParameter("zip_overwrite"), false);
	boolean keepZip = unpackZip && Values.getBoolean(upload.getParameter("zip_keep"), true);
	int numUnpackedFiles = 0;

	if (uploadedFile != null)
	{
		File f = null;
		OutputStream toFile = null;

		// Set file properties, check for duplicate files, save new file etc.
		if (!isZipFile || !unpackZip || keepZip)
		{
			f = (File)cc.getObject("item");
			boolean fileExists = false;
			boolean stopIfFileExists = true;
			
			if (f == null)
			{	
				String path = directory.getPath().toString() + "/" + uploadedFile.getFilename();
				f = File.getByPath(dc, new Path(path, Path.Type.FILE), true);
				if (f.isInDatabase())
				{
					fileExists = true;
					if (f.isRemoved() || (isZipFile && overwriteZip))
					{
						stopIfFileExists = false;
						f.setRemoved(false);
					}
				}
				else
				{
					dc.saveItem(f);
				}
			}
			else
			{
				stopIfFileExists = !f.getName().equals(uploadedFile.getFilename());
				if (stopIfFileExists) fileExists = File.exists(dc, directory, uploadedFile.getFilename());
				dc.reattachItem(f);
			}
			if (fileExists && stopIfFileExists)
			{
				throw new WebException("popup", "File already exists", "Another file with the same name already exists: {1}", uploadedFile.getFilename());
			}
			f.setName(uploadedFile.getFilename());
			String serverMimeType = application.getMimeType(uploadedFile.getFilename());
			String browserMimeType = uploadedFile.getMimeType();
			f.setMimeTypeAuto(serverMimeType != null ? serverMimeType : browserMimeType, null);
			if (fileTypeId >= 0) // < 0 = denied or unchanged
			{
				FileType ft = fileTypeId == 0 ? null : FileType.getById(dc, fileTypeId);
				f.setFileType(ft);
				if (ft != null) cc.setRecent(ft, maxRecent);
			}
			f.setDescription(description);
			toFile = f.getUploadStream(checkMd5, compress);
			f.setWriteProtected(writeProtected);
		}
		
		if (isZipFile && unpackZip)
		{
			PluginDefinition plugin = PluginDefinition.getById(dc, zipFormat);
			unpacker = plugin.newInstance(FileUnpacker.class, null, sc, null, null);
		}
		
		// We don't need the database connection while uploading
		dc.disconnect();
		
		InputStream uploadStream = uploadedFile.getInputStream();
		if (unpacker != null)
		{
			long fileSize = progress.getTotalBytes() - progress.getTransferredBytes();
			InputStreamSplitter splitter = null;
			if (toFile != null) 
			{
				uploadStream = new InputStreamSplitter(uploadStream, true, true, toFile);
			}
			
			SimpleAbsoluteProgressReporter unpackProgress = new SimpleAbsoluteProgressReporter(null, fileSize);
			sc.setSessionSetting("UnpackProgress", unpackProgress);
			numUnpackedFiles = unpacker.unpack(dc, directory, uploadStream, overwriteZip, unpackProgress);
		}
		else
		{
			FileUtil.copy(uploadStream, toFile);
		}
		uploadStream.close();
			
		// This will force the upload to continue reading the form data
		// in case it contains info that we need
		upload.next();
			
		// The upload is done, now set other properties
		dc.reconnect();
	}

	// The upload could not be finished for some reason, for example: the user closed
	// the browser window or pulled the network cable, or...
	if (progress.getTransferredBytes() != progress.getTotalBytes())
	{
		throw new UploadAbortedException();
	}
	dc.commit();
	message = unpackZip ? numUnpackedFiles + " file(s) unpacked successfully" : "File uploaded successfully";
	onload="done()";
}
catch (Throwable t)
{
	if (upload != null) upload.setException(t);
	if (t instanceof UploadAbortedException)
	{
		message = "The upload was aborted before it was completed.";
		onload="done()";
	}
	else
	{
		onload = "exception()";
	}
}
finally
{
	if (unpacker != null) unpacker.done();
	if (dc != null) dc.close();
}
%>
<html>
<head>
	<script language="JavaScript">
	function done()
	{
		window.top.location = '../../common/close_popup.jsp?refresh_opener=1&message=<%=HTML.urlEncode(message)%>';
	}
	function exception()
	{
		window.top.frames['select'].location = 'show_exception.jsp?ID=<%=ID%>';
	}
	</script>
</head>
<body onload="<%=onload%>">
</body>
</html>



